/*
   Copyright (C) 2013-2020 Laurent Montel <montel@kde.org>

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public
   License as published by the Free Software Foundation; either
   version 2 of the License, or (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; see the file COPYING.  If not, write to
   the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.
*/

#ifndef SENDLATERUTIL_H
#define SENDLATERUTIL_H

#include "sendlater_export.h"
#include <KSharedConfig>

namespace SendLater {
class SendLaterInfo;

/** Send later utilities. */
namespace SendLaterUtil {
Q_REQUIRED_RESULT SENDLATER_EXPORT bool compareSendLaterInfo(SendLater::SendLaterInfo *left, SendLater::SendLaterInfo *right);

SENDLATER_EXPORT KSharedConfig::Ptr defaultConfig();

SENDLATER_EXPORT void writeSendLaterInfo(KSharedConfig::Ptr config, SendLater::SendLaterInfo *info, bool forceReload = true);

Q_REQUIRED_RESULT SENDLATER_EXPORT bool sentLaterAgentWasRegistered();

Q_REQUIRED_RESULT SENDLATER_EXPORT bool sentLaterAgentEnabled();

SENDLATER_EXPORT void reload();

SENDLATER_EXPORT void changeRecurrentDate(SendLater::SendLaterInfo *info);
SENDLATER_EXPORT void forceReparseConfiguration();

Q_REQUIRED_RESULT SENDLATER_EXPORT QString sendLaterPattern();
SENDLATER_EXPORT void removeItem(qint64 item);
}
}
#endif // SENDLATERUTIL_H
